#include <Roster.h>
#include <File.h>
#include <Resources.h>
#include <Application.h>
#include "Global.h"


TPreference *PreferenceWindow;
TAuthorization *AuthorizationWindow;
TPassword *PasswordWindow;

// 自身の実行ファイルから GuiBerry リソースを抽出して BGView に再生する
status_t GetBGView(const char *RsrcName, BView **BGView)
{
	status_t status;
	
	// 実行ファイルの情報(app_info)を取得
	app_info appinfo;
	be_app->GetAppInfo(&appinfo);
	
	// 実行ファイル(appinfo.ref)をオープン
	BFile file(&appinfo.ref, B_READ_ONLY);
	status = file.InitCheck();
	if (status != B_OK) {
		return B_ERROR;
	}
	
	// file からリソースを抽出
	BResources rsrc(&file);
	size_t size;
	char *data = (char *)rsrc.LoadResource('ARCV', RsrcName, &size);
	if (data == NULL) {
		return B_ERROR;
	}
	
	// GuiBerry リソースは BMessage をアーカイブしたものなので
	// それを archive に展開(Unflatten)
	BMessage archive;
	status = archive.Unflatten(data);
	if (status != B_OK) {
		return B_ERROR;
	}
	
	// Unflatten されたデータを instantiate_object() に喰わせると
	// 最上位層の BView を取得できる。(それは BGView のはず)
	// instantiate_object() は、また同時にすべての階層の(子、孫、曾孫・・・の）
	// View を再生してくれる。
	// あとは、それを Window に AddChild() するだけで再現される。
	*BGView = dynamic_cast<BView *>(instantiate_object(&archive));
	if (BGView == NULL) {
		return B_ERROR;
	}
	
	return B_OK;
}
