#include <Application.h>
#include <Directory.h>
#include <Path.h>
#include <Alert.h>
#include "driver_settings.h"

#include "Global.h"
#include "Preference.h"


#define SETTINGS_PATH "/boot/home/config/settings/netbfs"
#define SETTINGS_FILE "netbfs.conf"

#define _FIND_VIEW(n, o, t)				\
	if (strcmp(v->Name(), n) == 0) {	\
		pref->o = (t *)v;				\
	}

void find_view(BView *bg, TPreference *pref)
{
	for(int i=0; i < bg->CountChildren(); i++) {
		BView *v = bg->ChildAt(i);
		_FIND_VIEW("Tab", Tab, BTabView);
		_FIND_VIEW("OKButton", OKButton, BButton);
		_FIND_VIEW("LogFile", LogFile, BTextControl);
		_FIND_VIEW("Comment", Comment, BTextControl);
		_FIND_VIEW("Browsable", Browsable, BCheckBox);
		_FIND_VIEW("GlWritable", GlWritable, BCheckBox);
		_FIND_VIEW("GlGuest", GlGuest, BCheckBox);
		_FIND_VIEW("UserList", UserList, BListView);
		_FIND_VIEW("NewUser", NewUser, BButton);
		_FIND_VIEW("RemoveUser", RemoveUser, BButton);
		_FIND_VIEW("ChangePasswd", ChangePasswd, BButton);
		_FIND_VIEW("ShareMenu", ShareMenu, BMenuField);
		_FIND_VIEW("ShareName", ShareName, BTextControl);
		_FIND_VIEW("NewShare", NewShare, BButton);
		_FIND_VIEW("RemoveShare", RemoveShare, BButton);
		_FIND_VIEW("ApplyShare", ApplyShare, BButton);
		_FIND_VIEW("SharePath", SharePath, BTextControl);
		_FIND_VIEW("Users", Users, BTextControl);
		_FIND_VIEW("Writable", Writable, BCheckBox);
		_FIND_VIEW("Guest", Guest, BCheckBox);
		if (dynamic_cast<BTabView *>(v)) {
			BView *tab;
			for(int j=0; ((BTabView *)v)->TabAt(j) != NULL; j++) {
				tab = ((BTabView *)v)->TabAt(j)->View();
				find_view(tab, pref);
			}
		} else {
			find_view(v, pref);
		}
	}
}

// TPreference

TPreference::TPreference(BRect frame)
	:	BWindow(frame, "NetBFS Server Preference", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	fInitCheck = false;
	
	if (GetBGView("Preference", &BGView) != B_OK) return;
	ResizeTo(BGView->Frame().Width(), BGView->Frame().Height());
	AddChild(BGView);
	find_view(BGView, this);
	
	// BScrollView と BMenuField は、Instantiate 後の挙動に不具合があるので、これだけは自前で作成する :(
	// BScrollView + BListView
	UserList = new BListView(BRect(7,10,225,172), "UserList",
		B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL_SIDES);
	BScrollView *scrollView = new BScrollView("UserListScrollView",
		UserList, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_FRAME_EVENTS, false, true);
	Tab->TabAt(1)->View()->AddChild(scrollView);
	
	// BMenuField
	ShareMenu = new BMenuField(BRect(17, 14, 100, 20), "ShareMenu", "Share", new BPopUpMenu("(Select)"));
	ShareMenu->SetDivider(40);
	Tab->TabAt(2)->View()->AddChild(ShareMenu);
	
	OKButton->SetMessage(new BMessage(OK_BTN_CLICKED));
	NewUser->SetMessage(new BMessage(NEW_USER_CLICKED));
	RemoveUser->SetMessage(new BMessage(REMOVE_USER_CLICKED));
	ChangePasswd->SetMessage(new BMessage(CHANGE_PASSWD_CLICKED));
	NewShare->SetMessage(new BMessage(NEW_SHARE_CLICKED));
	RemoveShare->SetMessage(new BMessage(REMOVE_SHARE_CLICKED));
	ApplyShare->SetMessage(new BMessage(APPLY_SHARE_CLICKED));
	
	Tab->Select(Tab->TabAt(0));
	
	fInitCheck = true;
}

TPreference::~TPreference()
{
	for(int i = UserList->CountItems() - 1; i >= 0; i--) {
		TUserListItem *item = (TUserListItem *)UserList->RemoveItem(i);
		delete item;
	}
}

bool TPreference::QuitRequested()
{
	be_app_messenger.SendMessage(B_QUIT_REQUESTED);
	return true;
}

void TPreference::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case OK_BTN_CLICKED:		SaveSettings(); break;
		case SHAREMENU_SELECTED:	ShareMenuSelected(msg); break;
		case NEW_USER_CLICKED:		NewUserClicked(); break;
		case REMOVE_USER_CLICKED:	RemoveUserClicked(); break;
		case CHANGE_PASSWD_CLICKED:	ChangePasswdClicked(); break;
		case NEW_SHARE_CLICKED:		NewShareClicked(); break;
		case REMOVE_SHARE_CLICKED:	RemoveShareClicked(); break;
		case APPLY_SHARE_CLICKED:	ApplyShareClicked(); break;
		case AUTHORIZED:			LoadSettings(); break;
		default: BWindow::MessageReceived(msg);
	}
}

void TPreference::LoadSettings()
{
	LogFile->SetText("/var/log/netbfsd.log");
	
	create_directory(SETTINGS_PATH, 0755);
	BPath path(SETTINGS_PATH, SETTINGS_FILE);
	void *handle = (void *)load_driver_settings_from_path(path.Path());
	if (handle == NULL) return;
	
	const struct driver_settings *settings = get_driver_settings(handle);
	struct driver_parameter *parameter = settings->parameters;
	
	for(int i = 0; i < settings->parameter_count; i++) {
		printf("name = %s  parameter_count = %d\n", parameter->name, parameter->parameter_count);
		if (strcmp(parameter->name, "global") == 0) {
			driver_parameter *global = parameter->parameters;
			for(int j = 0; j < parameter->parameter_count; j++) {
				printf("  %s = %s\n", global->name, *global->values);
				if (strcasecmp(global->name, "logfile") == 0) {
					LogFile->SetText(*global->values);
				} else if (strcasecmp(global->name, "comment") == 0) {
					Comment->SetText(*global->values);
				} else if (strcasecmp(global->name, "browsable") == 0) {
					Browsable->SetValue(!strcasecmp("yes", *global->values));
				} else if (strcasecmp(global->name, "writable") == 0) {
					GlWritable->SetValue(!strcasecmp("yes", *global->values));
				} else if (strcasecmp(global->name, "guest_ok") == 0) {
					GlGuest->SetValue(!strcasecmp("yes", *global->values));
				}
				global++;
			}
		} else if (strcmp(parameter->name, "password") == 0) {
			driver_parameter *passwd = parameter->parameters;
			for(int j = 0; j < parameter->parameter_count; j++) {
				printf("  %s = %s\n", passwd->name, *passwd->values);
				TUserListItem *item = new TUserListItem(passwd->name, *passwd->values);
				UserList->AddItem(item);
				passwd++;
			}
		} else {
			const char *sharePath = NULL, *users = "", *guestok = "no", *writable = "no";
			driver_parameter *share = parameter->parameters;
			for(int j = 0; j < parameter->parameter_count; j++) {
				printf("  %s = %s\n", share->name, *share->values);
				if (strcasecmp(share->name, "path") == 0) {
					sharePath = *share->values;
				} else if (strcasecmp(share->name, "user") == 0) {
					users = *share->values;
				} else if (strcasecmp(share->name, "writable") == 0) {
					writable = *share->values;
				} else if (strcasecmp(share->name, "guest_ok") == 0) {
					guestok = *share->values;
				}
				share++;
			}
			if (sharePath && users) {
				TShareMenuItem *item = new TShareMenuItem(parameter->name, sharePath, users,
					strcasecmp(writable, "yes") == 0, strcasecmp(guestok, "yes") == 0,
					new BMessage(SHAREMENU_SELECTED));
				ShareMenu->Menu()->AddItem(item);
			}
		}
		parameter++;
	}
	
	unload_driver_settings(handle);
}

void TPreference::SaveSettings()
{
	BString data;
	
	data << "global {\n";
	data << "	logfile = \"" << LogFile->Text() << "\"\n";
	data << "	comment = \"" << Comment->Text() << "\"\n";
	data << "	browsable = " << (Browsable->Value() ? "yes" : "no") << "\n";
	data << "	writable = " << (GlWritable->Value() ? "yes" : "no") << "\n";
	data << "	guest_ok = " << (GlGuest->Value() ? "yes" : "no") << "\n";
	data << "}\n\npassword {\n";
	for(int i = 0; i < UserList->CountItems(); i++) {
		TUserListItem *item = (TUserListItem *)UserList->ItemAt(i);
		data << "	\"" << item->Text() << "\" = \"" << item->Password() << "\"\n";
	}
	data << "}\n\n";
	for(int i = 0; i < ShareMenu->Menu()->CountItems(); i++) {
		TShareMenuItem *item = (TShareMenuItem *)ShareMenu->Menu()->ItemAt(i);
		data << "\"" << item->Label() << "\" {\n";
		data << "	path = \"" << item->SharePath() << "\"\n";
		data << "	user = \"" << item->Users() << "\"\n";
		data << "	writable = " << (item->IsWritable() ? "yes" : "no") << "\n";
		data << "	guest_ok = " << (item->IsGuestOk() ? "yes" : "no") << "\n";
		data << "}\n\n";
	}
	
	BString fname(SETTINGS_PATH);
	fname << "/" << SETTINGS_FILE;
	FILE *file = fopen(fname.String(), "w");
	if (file == NULL) {
		int err = errno;
		(new BAlert("", strerror(err), "OK"))->Go();
		return;
	}
	fwrite(data.String(), data.Length(), 1, file);
	fclose(file);
}

void TPreference::ShareMenuSelected(TShareMenuItem *item)
{
	if (item) {
		ShareName->SetText(item->Label());
		SharePath->SetText(item->SharePath());
		Users->SetText(item->Users());
		Writable->SetValue(item->IsWritable());
		Guest->SetValue(item->IsGuestOk());
		RemoveShare->SetEnabled(true);
		ShareName->SetEnabled(true);
		SharePath->SetEnabled(true);
		Users->SetEnabled(true);
		Writable->SetEnabled(true);
		Guest->SetEnabled(true);
		ApplyShare->SetEnabled(true);
	} else {
		ShareName->SetText("");
		SharePath->SetText("");
		Users->SetText("");
		Writable->SetValue(0);
		Guest->SetValue(0);
		RemoveShare->SetEnabled(false);
		ShareName->SetEnabled(false);
		SharePath->SetEnabled(false);
		Users->SetEnabled(false);
		Writable->SetEnabled(false);
		Guest->SetEnabled(false);
		ApplyShare->SetEnabled(false);
	}
}

void TPreference::ShareMenuSelected(BMessage *msg)
{
	void *p;
	TShareMenuItem *share;
	if (msg->FindPointer("source", &p) != B_OK) return;
	share = reinterpret_cast<TShareMenuItem *>(p);
	ShareMenuSelected(share);
}

bool TPreference::ChangePassword(TUserListItem *item)
{
	int icheck;
	PasswordWindow->Lock();
	PasswordWindow->Open(item->Text());
	PasswordWindow->Unlock();
	while ((icheck = PasswordWindow->InputCheck()) == 0) {
		snooze(10000);
		this->UpdateIfNeeded();
	}
	if (icheck != 1) return false;
	PasswordWindow->Lock();
	item->SetText(PasswordWindow->NewName());
	item->SetPassword(PasswordWindow->NewPassword());
	PasswordWindow->Unlock();
	return true;
}

void TPreference::NewUserClicked()
{
	TUserListItem *item = new TUserListItem("NewUser", "");
	if (ChangePassword(item)) {
		UserList->AddItem(item);
	} else {
		delete item;
	}
}

void TPreference::RemoveUserClicked()
{
	int32 i = UserList->CurrentSelection();
	if (i >= 0) {
		TUserListItem *item = (TUserListItem *)UserList->RemoveItem(i);
		delete item;
	}
}

void TPreference::ChangePasswdClicked()
{
	int32 i = UserList->CurrentSelection();
	if (i >= 0) ChangePassword((TUserListItem *)UserList->ItemAt(i));
}

void TPreference::NewShareClicked()
{
	TShareMenuItem *item = new TShareMenuItem("New shared folder", "/boot/home", "",
							false, false, new BMessage(SHAREMENU_SELECTED));
	ShareMenu->Menu()->AddItem(item);
	item->SetMarked(true);
	ShareMenuSelected(item);
}

void TPreference::RemoveShareClicked()
{
	TShareMenuItem *item = (TShareMenuItem *)ShareMenu->Menu()->FindMarked();
	if (item == NULL) return;
	ShareMenu->Menu()->RemoveItem(item);
	ShareMenu->MenuItem()->SetLabel("(Select)");
	ShareMenuSelected((TShareMenuItem *)NULL);
}

void TPreference::ApplyShareClicked()
{
	TShareMenuItem *item = (TShareMenuItem *)ShareMenu->Menu()->FindMarked();
	if (item == NULL) return;
	
	item->SetLabel(ShareName->Text());
	item->SetSharePath(SharePath->Text());
	item->SetUsers(Users->Text());
	item->SetWritable(Writable->Value());
	item->SetGuestOk(Guest->Value());
	
	ShareMenu->MenuItem()->SetLabel(ShareName->Text());
}

// TUserListItem

TUserListItem::TUserListItem(const char *userName, const char *passwd)
	:	BStringItem(userName)
{
	fPasswd.SetTo(passwd);
}

TUserListItem::~TUserListItem()
{
}

const char *TUserListItem::Password() const
{
	return fPasswd.String();
}

// TShareMenuItem

TShareMenuItem::TShareMenuItem(const char *shareName,
		const char *sharePath, const char *users,
		bool writable, bool guestOk, BMessage *msg)
	:	BMenuItem(shareName, msg)
{
	fSharePath.SetTo(sharePath);
	fUsers.SetTo(users);
	fWritable = writable;
	fGuestOk = guestOk;
}

TShareMenuItem::~TShareMenuItem()
{
	
}
