#ifndef _PREFERENCE_H
#define _PREFERENCE_H

#include <Window.h>
#include <View.h>
#include <Button.h>
#include <StringView.h>
#include <CheckBox.h>
#include <TextControl.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <interface/TabView.h>
#include <String.h>
#include <ListView.h>
#include <ScrollView.h>
#include <ListItem.h>
#include <PopUpMenu.h>

#include "driver_settings.h"

class TUserListItem : public BStringItem
{
public:
	TUserListItem(const char *userName, const char *passwd);
	~TUserListItem();
	const char *Password() const;
	void SetPassword(const char *passwd) { fPasswd.SetTo(passwd); };
private:
	BString fPasswd;
};

class TShareMenuItem : public BMenuItem
{
public:
	TShareMenuItem(const char *shareName,
			const char *sharePath, const char *users,
			bool writable, bool guestOk, BMessage *msg);
	~TShareMenuItem();
	const char *SharePath() const { return fSharePath.String(); };
	const char *Users() const { return fUsers.String(); };
	bool IsWritable() const { return fWritable; };
	bool IsGuestOk() const { return fGuestOk; };
	void SetSharePath(const char *path) { fSharePath.SetTo(path); };
	void SetUsers(const char *users) { fUsers.SetTo(users); };
	void SetWritable(bool writable) { fWritable = writable; };
	void SetGuestOk(bool guestok) { fGuestOk = guestok; };

private:
	BString fSharePath;
	BString fUsers;
	bool fWritable;
	bool fGuestOk;
};

class TPreference : public BWindow
{
public:
	TPreference(BRect frame);
	~TPreference();
	bool QuitRequested();
	void MessageReceived(BMessage *msg);
	bool InitCheck() const { return fInitCheck; };
	
	BView *BGView;
	BTabView *Tab;
	BButton *OKButton;
	
	BTextControl *LogFile;
	BTextControl *Comment;
	BCheckBox *Browsable;
	BCheckBox *GlWritable;
	BCheckBox *GlGuest;
	
	BListView *UserList;
	BButton *NewUser;
	BButton *RemoveUser;
	BButton *ChangePasswd;
	
	BMenuField *ShareMenu;
	BButton *NewShare;
	BButton *RemoveShare;
	BButton *ApplyShare;
	BTextControl *ShareName;
	BTextControl *SharePath;
	BTextControl *Users;
	BCheckBox *Writable;
	BCheckBox *Guest;
	
private:
	bool fInitCheck;
	void LoadSettings();
	void SaveSettings();
	void ShareMenuSelected(TShareMenuItem *item);
	void ShareMenuSelected(BMessage *msg);
	bool ChangePassword(TUserListItem *item);
	void NewUserClicked();
	void RemoveUserClicked();
	void ChangePasswdClicked();
	void NewShareClicked();
	void RemoveShareClicked();
	void ApplyShareClicked();
	
};


#endif
