#include <Alert.h>
#include "BrowserWindow.h"
#include "StrTok.h"
#include "StringEx.h"

enum {
	IPADRS_CLM = 1,
	VERSION_CLM,
	COMMENT_CLM,
};

enum {
	SHARE_NAME_CLM = 1,
	MOUNTEDAT_CLM,
};

BString mount_root;

int32 window_index(BWindow *window)
{
	if (window == NULL) return -1;
	for(int32 i = 0; i < be_app->CountWindows(); i++) if (be_app->WindowAt(i) == window) return i;
	return -1;
}

void get_name(int32 from, BString *name)
{
	name->SetTo("");
	*name << (uint32)((uint8 *)&from)[0] << "." <<
	 	(uint32)((uint8 *)&from)[1] << "." <<
	 	(uint32)((uint8 *)&from)[2] << "." <<
	 	(uint32)((uint8 *)&from)[3];
}

// TServerItem

TServerItem::TServerItem(const char *adrs)
	:	CLVEasyItem()
{
	fServerAdrs.SetTo(adrs);
	Window = NULL;
}

TServerItem::~TServerItem()
{
	if (Window != NULL) BMessenger(Window).SendMessage(B_QUIT_REQUESTED);
}

void TServerItem::ClearShareNames()
{
	for(int32 i = 0; i < fShareNameList.CountItems(); i++) {
		BString *item = (BString *)fShareNameList.ItemAt(i);
		delete item;
	}
	if (window_index(this->Window) >= 0) {
		this->Window->Lock();
		this->Window->ClearShareList();
		this->Window->Unlock();
	}
	fShareNameList.MakeEmpty();
}

void TServerItem::AddShareName(const char *name)
{
	fShareNameList.AddItem(new BString(name));
}

// TBrowserWindow

TBrowserWindow::TBrowserWindow(BRect rect)
	:	BWindow(rect, "NetBFS Servers", B_DOCUMENT_WINDOW, 0)
{
	mount_root.SetTo("/netbfs/");
	
	CLVContainerView *containerView;
	BRect rect(Bounds());
	rect.right -= B_V_SCROLL_BAR_WIDTH;
	rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
	fColumnListView = new ColumnListView(rect, &containerView, "ColumnListView", B_FOLLOW_ALL_SIDES);
	fColumnListView->AddColumn(new CLVColumn(NULL, 20.0, CLV_LOCK_AT_BEGINNING | CLV_NOT_MOVABLE));
	fColumnListView->AddColumn(new CLVColumn("IP Address", 80, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50));
	fColumnListView->AddColumn(new CLVColumn("Version", 80, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50));
	fColumnListView->AddColumn(new CLVColumn("Comment", 130, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50));
	AddChild(containerView);
	fColumnListView->SetTarget(this);
	fColumnListView->SetInvocationMessage(new BMessage(SELECTED_MSG));
	
	struct sockaddr_in addr;
	fSocket = socket(AF_INET, SOCK_DGRAM, 0);
	if (fSocket < 0) {
		(new BAlert("", "socket() error.\n", ""))->Go();
		be_app_messenger.SendMessage(B_QUIT_REQUESTED);
		return;
	}
	addr.sin_family = AF_INET;
	addr.sin_port = htons(4991);
	addr.sin_addr.s_addr = htonl(INADDR_ANY);
	if (bind(fSocket, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
		(new BAlert("", "bind() error.\n", ""))->Go();
		be_app_messenger.SendMessage(B_QUIT_REQUESTED);
		return;
	}
	
	static recv_thread_t data;
	data.socket = fSocket;
	data.looper = this;
	fRecvThreadID = spawn_thread(recv_thread, "recv_thread", B_LOW_PRIORITY, &data);
	if (fRecvThreadID < 0) {
		(new BAlert("", "spawn_thread() failed.\n", ""))->Go();
		be_app_messenger.SendMessage(B_QUIT_REQUESTED);
		return;
	}
	if (resume_thread(fRecvThreadID) != B_OK) {
		(new BAlert("", "resume_thread() failed.\n", ""))->Go();
		be_app_messenger.SendMessage(B_QUIT_REQUESTED);
		return;
	}
	
	fSBuff = new TStreamBuffer(SB_UNKNOWN_TEXT_TYPE, 0);
}

TBrowserWindow::~TBrowserWindow()
{
	kill_thread(fRecvThreadID);
	close(fSocket);
	delete fSBuff;
}

bool TBrowserWindow::QuitRequested()
{
	be_app_messenger.SendMessage(B_QUIT_REQUESTED);
	return true;
}

void TBrowserWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case RECEIVED_MSG:
			Received(msg);
			break;
		case SELECTED_MSG:
			Selected(msg);
			break;
		default: BLooper::MessageReceived(msg);
	}
}

TServerItem *TBrowserWindow::AddServerItem(int32 from, const char *ver, const char *comment)
{
	BString name;
	get_name(from, &name);
	
	TServerItem *item = new TServerItem(name.String());
	item->SetColumnContent(IPADRS_CLM, name.String());
	item->SetColumnContent(VERSION_CLM, ver);
	item->SetColumnContent(COMMENT_CLM, comment);
	fColumnListView->AddItem(item);
	return item;
}

TServerItem *TBrowserWindow::UpdateServerItem(TServerItem *item, const char *ver, const char *comment)
{
	if (item == NULL) return NULL;
	item->SetColumnContent(VERSION_CLM, ver);
	item->SetColumnContent(COMMENT_CLM, comment);
	return item;
}

TServerItem *TBrowserWindow::FindServerItem(const char *from) const
{
	for(int32 i=0; i < fColumnListView->CountItems(); i++) {
		TServerItem *item = (TServerItem *)fColumnListView->ItemAt(i);
		if (strcmp(item->ServerAdrs(), from) == 0) return item;
	}
	return NULL;
}

void TBrowserWindow::Received(BMessage *msg)
{
	int32 from;
	const char *str;
	if ((msg->FindInt32("from", &from) != B_OK) || (msg->FindString("msg", &str))) return;
	
	printf("received %lu bytes from %u.%u.%u.%u %s", strlen(str),
		((uint8 *)&from)[0], ((uint8 *)&from)[1], ((uint8 *)&from)[2], ((uint8 *)&from)[3], str);
	fflush(stdout);
	
	fSBuff->AddStream(str);
	BString sdata;
	while(fSBuff->OutputStream(&sdata) >= 0) {
		TStrTok strtok(sdata.String(), ",", TOKEN_COUNT_NULLSTR);
		if (strtok.CountTokens() < 3) continue;
		const char *ver = strtok.Token(0);
		const char *command = strtok.Token(1);
		const char *comment = strtok.Token(2);
		BString name;
		get_name(from, &name);
		TServerItem *item = FindServerItem(name.String());
		if (strcmp(command, "HELLO") == 0) {
			if (item == NULL) item = AddServerItem(from, ver, comment); else UpdateServerItem(item, ver, comment);
			if (item != NULL) {
				item->ClearShareNames();
				for(int32 i = 3; i < strtok.CountTokens(); i++) item->AddShareName(strtok.Token(i));
			}
		} else if (strcmp(command, "BYE") == 0) {
			if (item != 0) {
				fColumnListView->RemoveItem(item);
				delete item;
			}
		}
	}
}

void TBrowserWindow::Selected(BMessage *msg)
{
	int32 index;
	if (msg->FindInt32("index", &index) != B_OK) return;
	TServerItem *item = (TServerItem *)fColumnListView->ItemAt(index);
	if (window_index(item->Window) < 0) {
		item->Window = new TShareListWindow(BRect(250, 150, 580, 350),
			item->GetColumnContentText(IPADRS_CLM), item->ServerAdrs(), item->ShareNameList());
	}
	item->Window->Show();
	item->Window->Activate();
}

// recv_thread

int32 recv_thread(void *data)
{
	char buff[1024];
	int fromlen, rsize;
	struct sockaddr_in from;
	int sock = ((recv_thread_t *)data)->socket;
	BMessenger msgr(((recv_thread_t *)data)->looper);
	
	while(true) {
		fromlen = sizeof(from);
		rsize = recvfrom(sock, buff, sizeof(buff), 0, (struct sockaddr *)&from, &fromlen);
		BString str(buff, rsize);
		BMessage msg(RECEIVED_MSG);
		msg.AddInt32("from", from.sin_addr.s_addr);
		msg.AddString("msg", str.String());
		msgr.SendMessage(msg);
	}
	return 0;
}

// TShareListWindow

TShareListWindow::TShareListWindow(BRect rect, const char *title, const char *server, const BList *shareNames)
	:	BWindow(rect, title, B_DOCUMENT_WINDOW, 0)
{
	Server.SetTo(server);
	CLVContainerView *containerView;
	BRect rect(Bounds());
	rect.right -= B_V_SCROLL_BAR_WIDTH;
	rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
	fShareListView = new ColumnListView(rect, &containerView, "ColumnListView", B_FOLLOW_ALL_SIDES);
	fShareListView->AddColumn(new CLVColumn(NULL, 20.0, CLV_LOCK_AT_BEGINNING | CLV_NOT_MOVABLE));
	fShareListView->AddColumn(new CLVColumn("Name", 80, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50));
	fShareListView->AddColumn(new CLVColumn("Mounted at", 160, CLV_HEADER_TRUNCATE | CLV_TELL_ITEMS_WIDTH, 50));
	AddChild(containerView);
	fShareListView->SetTarget(this);
	fShareListView->SetInvocationMessage(new BMessage(SELECTED_MSG));
	
	AddShareItem(shareNames);
}

TShareListWindow::~TShareListWindow()
{
}

void TShareListWindow::AddShareItem(const BList *list)
{
	fShareListView->MakeEmpty();
	for(int32 i = 0; i < list->CountItems(); i++) {
		BString *str = (BString *)list->ItemAt(i);
		AddShareItem(str->String());
	}
	fShareListView->Invalidate();
}

void TShareListWindow::AddShareItem(const char *shareName)
{
	TShareItem *item = new TShareItem();
	item->SetColumnContent(1, shareName);
	fShareListView->AddItem(item);
}

void TShareListWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case SELECTED_MSG:
			Mount(msg);
			break;
		default: BWindow::MessageReceived(msg);
	}
}

void TShareListWindow::ClearShareList()
{
	for(int32 i = fShareListView->CountItems() - 1 ; i > 0; i++) {
		TShareItem *item = (TShareItem *)fShareListView->ItemAt(i);
		fShareListView->RemoveItem(i);
		delete item;
	}
}

void TShareListWindow::Mount(BMessage *msg)
{
	int32 index;
	if (msg->FindInt32("index", &index) != B_OK) return;
	
	TShareItem *item = (TShareItem *)fShareListView->ItemAt(index);
	const char *shareName = item->GetColumnContentText(SHARE_NAME_CLM);
	if (window_index(item->Dialog) < 0) {
		BString mountPoint(mount_root);
		mountPoint << Title() << "/" << shareName;
		BString title(Title());
		title << "->" << shareName;
		item->Dialog = new TMountDialog(BRect(300, 200, 550, 315), title.String(),
			Server.String(), shareName, mountPoint.String());
	}
	item->Dialog->Show();
	item->Dialog->Activate();
}
