#ifndef _BROWSER_WINDOW_H
#define _BROWSER_WINDOW_H

#include <Application.h>
#include <Window.h>
#include <stdio.h>
#include <socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <String.h>
#include <List.h>
#include "MountDialog.h"
#include "CLVEasyItem.h"
#include "ColumnListView.h"
#include "StreamBuffer.h"


enum {
	RECEIVED_MSG = 'recv',
	SELECTED_MSG = 'sele',
};

typedef struct recv_thread_t {
	int socket;
	BLooper *looper;
} recv_thread_t;


class TShareListWindow;

// TServerItem

class TServerItem : public CLVEasyItem
{
public:
	TServerItem(const char *adrs);
	~TServerItem();
	const char *ServerAdrs() const { return fServerAdrs.String(); };
	void ClearShareNames();
	void AddShareName(const char *name);
	int32 CountShares() const { return fShareNameList.CountItems(); };
	const char *ShareNameAt(int32 index) const { return ((BString *)fShareNameList.ItemAt(index))->String(); };
	const BList *ShareNameList() const { return &fShareNameList; };
	TShareListWindow *Window;
private:
	BString fServerAdrs;
	BList fShareNameList;
};

// TBrowserWindow

class TBrowserWindow : public BWindow
{
public:
	TBrowserWindow(BRect rect);
	~TBrowserWindow();
	bool QuitRequested();
	void MessageReceived(BMessage *msg);
	
private:
	ColumnListView *fColumnListView;
	int fSocket;
	thread_id fRecvThreadID;
	bool fContinue;
	TStreamBuffer *fSBuff;
	
	TServerItem *AddServerItem(int32 from, const char *ver, const char *comment);
	TServerItem *UpdateServerItem(TServerItem *item, const char *ver, const char *comment);
	TServerItem *FindServerItem(const char *from) const;
	void Received(BMessage *msg);
	void Selected(BMessage *msg);
};

// TShareItem

class TShareItem : public CLVEasyItem
{
public:
	TShareItem() : CLVEasyItem() { Dialog = NULL; };
	TMountDialog *Dialog;
private:
	
};

// TShareListWindow

class TShareListWindow : public BWindow
{
public:
	TShareListWindow(BRect rect, const char *title, const char *server, const BList *shareNames);
	~TShareListWindow();
	void AddShareItem(const BList *list);
	void AddShareItem(const char *shareName);
	void MessageReceived(BMessage *msg);
	void ClearShareList();
private:
	ColumnListView *fShareListView;
	BString Server;
	void Mount(BMessage *msg);
};


int32 recv_thread(void *data);

#endif
