#include <Alert.h>
#include <stdio.h>
#include <string.h>
#include "MountDialog.h"
#include "mount_netbfs.h"


TMountDialog::TMountDialog(BRect rect, const char *title,
		const char *server, const char *shareName, const char *mountPoint)
	:	BWindow(rect, title, B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE)
{
	Server.SetTo(server);
	ShareName.SetTo(shareName);
	MountPoint.SetTo(mountPoint);
	
	BView *bgview = new BView(Bounds(), "bgview", 0, 0);
	bgview->SetViewColor(0xd8, 0xd8, 0xd8);
	AddChild(bgview);
	
	UserName = new BTextControl(BRect(12, 8, 242, 31), "UserName", "User Name", "", NULL);
	Password =  new BTextControl(BRect(12, 30, 242, 49), "Password", "Password", "", NULL);
	Guest = new BCheckBox(BRect(14, 57, 240, 77), "Guest", "Guest Login", new BMessage('gues'));
	MountButton = new BButton(BRect(90, 84, 160, 110), "Mount", "Mount", new BMessage('munt'));
	CancelButton =  new BButton(BRect(172, 84, 242, 110), "Cancel", "Cancel", new BMessage(B_QUIT_REQUESTED));
	
	UserName->SetDivider(60);
	UserName->SetAlignment(B_ALIGN_RIGHT, B_ALIGN_LEFT);
	Password->SetDivider(60);
	Password->SetAlignment(B_ALIGN_RIGHT, B_ALIGN_LEFT);
	Password->TextView()->HideTyping(true);
	
	bgview->AddChild(UserName);
	bgview->AddChild(Password);
	bgview->AddChild(Guest);
	bgview->AddChild(MountButton);
	bgview->AddChild(CancelButton);
	
	UserName->MakeFocus();
	MountButton->MakeDefault(true);
}

TMountDialog::~TMountDialog()
{
}

void TMountDialog::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case 'gues': {
			bool guest = !Guest->Value();
			UserName->SetEnabled(guest);
			Password->SetEnabled(guest);
			break;
		}
		case 'munt': {
			Mount();
			break;
		}
		default: BWindow::MessageReceived(msg);
	}
}

void TMountDialog::Mount()
{
	printf("Mounting %s\n", MountPoint.String());
	
	const char *userName = UserName->Text();
	const char *password = Password->Text();
	
	int result = mount_netbfs(Server.String(), NETBFS_TCP_PORT, ShareName.String(), 
		MountPoint.String(), userName, password);
	
	if (result != 0) {
		BString err("Could not mount.\n");
		err << strerror(result);
		(new BAlert("", err.String(), "OK"))->Go();
	} else {
		BMessenger(this).SendMessage(B_QUIT_REQUESTED);
	}
}
