#include "dircache.h"

// dir を追加/更新・検索・削除
dir_cache_t *c_add_dir(dir_cache_t **first, const char *path, vnode_id id)
{
	dir_cache_t *dir;
	if (*first == NULL) {
		// new cache
		dir = (dir_cache_t *)calloc(sizeof(dir_cache_t), 1);
		*first = dir;
	} else {
		dir = c_find_dir(*first, id);
		if (dir == NULL) {
			// insert
			dir = (dir_cache_t *)calloc(sizeof(dir_cache_t), 1);
			(*first)->prev = dir;
			dir->next = *first;
			*first = dir;
		} else {
			// update
			free(dir->path);
		}
	}
	dir->path = (char *)calloc(strlen(path) + 1, 1);
	strcpy(dir->path, path);
	dir->vnid = id;
	return dir;
}

dir_cache_t *c_find_dir(dir_cache_t *first, vnode_id id)
{
	dir_cache_t *dir = first;
	while(dir != NULL) {
		if (dir->vnid == id) break;
		dir = dir->next;
	}
	return dir;
}

void c_remove_dir(dir_cache_t **first, vnode_id id)
{
	entry_cache_t *entry, *next;
	dir_cache_t *dir = c_find_dir(*first, id);
	if (dir == NULL) return;
	
	c_make_empty_entry(dir);
	if (dir->next != NULL) dir->next->prev = dir->prev;
	if (dir->prev != NULL) dir->prev->next = dir->next;
	if (dir == *first) *first = dir->next;
	
	free(dir->path);
	free(dir);
}

void c_make_empty_dir(dir_cache_t *first)
{
	dir_cache_t *next, *dir = first;
	while(dir != NULL) {
		c_make_empty_entry(dir);
		next = dir->next;
		free(dir);
		dir = next;
	}
}

// dir にエントリを追加/更新・検索・削除
entry_cache_t *c_add_entry(dir_cache_t *dir, const char *leafname, uint16 namelen, const struct stat *stat)
{
	entry_cache_t *fentry, *entry = c_find_entry_vnid(dir, stat->st_ino);
	if (entry == NULL) {
		// insert
		entry = (entry_cache_t *)calloc(sizeof(entry_cache_t), 1);
		fentry = dir->entries;
		if (fentry != NULL) fentry->prev = entry;
		entry->next = fentry;
		dir->entries = entry;
	} else {
		// update
		free(entry->leafname);
	}
	entry->leafname = (char *)calloc(namelen + 1, 1);
	strncpy(entry->leafname, leafname, namelen);
	entry->stat = *stat;
	return entry;
}

entry_cache_t *c_find_entry_vnid(dir_cache_t *dir, vnode_id id)
{
	entry_cache_t *entry = dir->entries;
	while(entry != NULL) {
		if (entry->stat.st_ino == id) break;
		entry = entry->next;
	}
	return entry;
}

entry_cache_t *c_find_entry_name(dir_cache_t *dir, const char *leafname)
{
	entry_cache_t *entry = dir->entries;
	while(entry != NULL) {
		if (strcmp(entry->leafname, leafname) == 0) break;
		entry = entry->next;
	}
	return entry;
}


void c_remove_entry(dir_cache_t *dir, vnode_id id)
{
	entry_cache_t *entry = c_find_entry_vnid(dir, id);
	if (entry == NULL) return;
	
	if (entry->next != NULL) entry->next->prev = entry->prev;
	if (entry->prev != NULL) entry->prev->next = entry->next;
	
	free(entry->leafname);
	free(entry);
}

void c_make_empty_entry(dir_cache_t *dir)
{
	entry_cache_t *next, *entry = dir->entries;
	while(entry != NULL) {
		next = entry->next;
		free(entry->leafname);
		free(entry);
		entry = next;
	}
}
