#include <stdio.h>
#include <socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <string.h>
#include <String.h>

#include "broadcasting.h"


typedef struct bs_t {
	BString data;
	int sock;
} bs_t;

struct sockaddr_in addr;
volatile bool canceled;
bs_t bs;

int32 broadcasting_thread(void *data)
{
	int s;
	const char *str = bs.data.String();
	int len = strlen(str);
	
	while (!canceled) {
		s = sendto(bs.sock, str, len, 0, (struct sockaddr *)&addr, sizeof(addr));
		if (s < 0) fprintf(stderr, "browse_server_thread: sendto() failed.\n");
		snooze(1000000);
	}
	return 0;
}

void start_broadcasting(const char *comment)
{
	int e, s, yes = 1;
	
	addr.sin_family = AF_INET;
	addr.sin_port = htons(4991);
	addr.sin_addr.s_addr = INADDR_BROADCAST;
	
	bs.data.SetTo(SERVER_EXISTS);
	bs.data << comment << "\n";
	
	bs.sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (bs.sock < 0) {
		e = errno;
		fprintf(stderr, "start_browse_server: socket() failed. %s\n", strerror(e));
		return;
	}
	s = setsockopt(bs.sock, SOL_SOCKET, SO_BROADCAST, &yes, sizeof(yes));
	if (s < 0) {
		e = errno;
		fprintf(stderr, "start_browse_server: setsockopt() failed. %s\n", strerror(e));
		close(bs.sock);
		return;
	}
	
	canceled = false;
	thread_id id = spawn_thread(broadcasting_thread, "broadcasting_thread", B_LOW_PRIORITY, NULL);
	if (id < 0) {
		fprintf(stderr, "Can't start broadcasting_thread. spawn_thread() failed.\n");
		return;
	}
	if (resume_thread(id) != B_OK) {
		fprintf(stderr, "Can't start broadcasting_thread. resume_thread() failed.\n");
		return;
	}
}

void stop_broadcasting()
{
	canceled = true;
	sendto(bs.sock, SERVER_SHUTTING_DOWN, strlen(SERVER_SHUTTING_DOWN), 0, (struct sockaddr *)&addr, sizeof(addr));
	close(bs.sock);
}
