#include "global.h"

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <strings.h>
#include "driver_settings.h"

ofstream LOG;


// TConfig (config loader)

TConfig::TConfig()
{
	fError = -1;
}

TConfig::~TConfig()
{
	Unload();
}

int TConfig::Load(const char *conf_file)
{
	void *handle = (void *)load_driver_settings_from_path(conf_file);
	if (handle == NULL) {
		fStrError.SetTo("Unable to load config file.");
		return -1;
	}
	
	const struct driver_settings *settings = get_driver_settings(handle);
	struct driver_parameter *parameter = settings->parameters;
	
	for(int i = 0; i < settings->parameter_count; i++) {
		if (strcmp(parameter->name, "global") == 0) {
			driver_parameter *global = parameter->parameters;
			for(int j = 0; j < parameter->parameter_count; j++) {
				if (strcasecmp(global->name, "logfile") == 0) {
					fLogFile.SetTo(*global->values);
				} else if (strcasecmp(global->name, "comment") == 0) {
					fComment.SetTo(*global->values);
				} else if (strcasecmp(global->name, "browsable") == 0) {
					fBrowsable = strcasecmp("yes", *global->values) == 0;
				} else if (strcasecmp(global->name, "writable") == 0) {
					fWritable = strcasecmp("yes", *global->values) == 0;
				} else if (strcasecmp(global->name, "guest_ok") == 0) {
					fGuestOk = strcasecmp("yes", *global->values) == 0;
				}
				global++;
			}
		} else if (strcmp(parameter->name, "password") == 0) {
			driver_parameter *passwd = parameter->parameters;
			for(int j = 0; j < parameter->parameter_count; j++) {
				user_t *item = (user_t *)malloc(sizeof(user_t));
				item->name = new BString(passwd->name);
				item->password = new BString(*passwd->values);
				fUserList.AddItem(item);
				passwd++;
			}
		} else {
			const char *sharePath = NULL, *users = "", *guestok = "no", *writable = "no";
			driver_parameter *share = parameter->parameters;
			for(int j = 0; j < parameter->parameter_count; j++) {
				if (strcasecmp(share->name, "path") == 0) {
					sharePath = *share->values;
				} else if (strcasecmp(share->name, "user") == 0) {
					users = *share->values;
				} else if (strcasecmp(share->name, "writable") == 0) {
					writable = *share->values;
				} else if (strcasecmp(share->name, "guest_ok") == 0) {
					guestok = *share->values;
				}
				share++;
			}
			if (sharePath && users) {
				share_folder_t *item = (share_folder_t *)malloc(sizeof(share_folder_t));
				item->name = new BString(parameter->name);
				item->path = new BString(sharePath);
				item->users = new BString(users),
				item->writable = strcasecmp(writable, "yes") == 0;
				item->guest_ok = strcasecmp(guestok, "yes") == 0;
				fShareList.AddItem(item);
			}
		}
		parameter++;
	}
	
	unload_driver_settings(handle);
	
	fError = 0;
	fStrError.SetTo("");

	return fError;
}

void TConfig::Unload()
{
	int32 i;
	for(i = 0; i < fUserList.CountItems(); i++) {
		user_t *user = (user_t *)fUserList.ItemAt(i);
		delete user->name;
		delete user->password;
		free(user);
	}
	for(i = 0; i < fShareList.CountItems(); i++) {
		share_folder_t *folder = (share_folder_t *)fShareList.ItemAt(i);
		delete folder->name;
		delete folder->path;
		delete folder->users;
		free(folder);
	}
	fError = -1;
}

const user_t *TConfig::FindUser(const char *userName) const
{
	for(int32 i = 0; i < fUserList.CountItems(); i++) {
		user_t *user = (user_t *)fUserList.ItemAt(i);
		if (strcmp(userName, user->name->String()) == 0) return user;
	}
	return NULL;
}

const share_folder_t *TConfig::FindShare(const char *shareName) const
{
	for(int32 i = 0; i < fShareList.CountItems(); i++) {
		share_folder_t *share = (share_folder_t *)fShareList.ItemAt(i);
		if (strcmp(shareName, share->name->String()) == 0) return share;
	}
	return NULL;
}

void TConfig::ShareNames(BString *names) const
{
	for(int32 i = 0; i < fShareList.CountItems(); i++) {
		if (names->Length() > 0) *names << ",";
		share_folder_t *item = (share_folder_t *)fShareList.ItemAt(i);
		*names << item->name->String();
	}
}



// dirname
char *dirname(char *path)
{
	char *newpath;
	char *slash;
	int length;
	
	slash = strrchr(path, '/');
	if (slash == 0) {
		path = ".";
		length = 1;
	} else {
		while (slash > path && *slash == '/') slash--;
		length = slash - path + 1;
	}
	newpath = (char *)malloc(length + 1);
	if (newpath == 0) return 0;
	strncpy (newpath, path, length);
	newpath[length] = 0;
	return newpath;
}

// basename
char *basename(char *path)
{
	char *base = (char *)strrchr(path, '/');
	return base ? base + 1 : path;
}

void dump_file(const char *name, const void *data, uint32 numBytes)
{
	FILE *fd = fopen(name, "w");
	fwrite(data, numBytes, 1, fd);
	fclose(fd);
}
