#include <sys/socket.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <netdb.h>
#include <signal.h>
#include <lzo1x.h>

#include "global.h"
#include "nb_config.h"
#include "client.h"
#include "netbfs.h"
#include "broadcasting.h"


int listening;
TConfig config;


// prototype
int main(int argc, char *argv[]);
void signal_handler(int sig);
int start_server();


// implementation
int main(int argc, char *argv[])
{
	int status = 0;
	bool inetd_mode = false;
	
	if (config.Load(CONFIG_FILE) != 0) {
		fprintf(stderr, config.StrError());
		fprintf(stderr, "\n");
		return -1;
	}
	
	if (lzo_init() != LZO_E_OK) {
		fprintf(stderr, "lzo_init() failed.\n");
		return -1;
	}
	
//	LOG.open("/dev/null");
	LOG.open(config.LogFile());
	
	if (argc > 1) {
		if (strcmp(argv[1], "inetd") == 0) inetd_mode = true;
	}
	
	if (inetd_mode) {
		// inetd からの起動
		TClient *client = new TClient(-1, NULL, &config);
		client->run();
		delete client;
	} else {
		// 単独デーモンとして、接続待機
		status = start_server();
	}
	
	LOG.close();
	return status;
}

void signal_handler(int sig)
{
	fprintf(stderr, "Signal %d catched.\n", sig);
	if (sig == SIGINT) listening = false;
}

#ifdef _OSTYPE_BEOS
// BeOS -------------------------------------------------------------------------

#include <be/kernel/OS.h>
#include <String.h>

typedef struct _data_t {
	int fd;
	struct sockaddr_in cli_addr;
} _data_t;

int32 _accepted_thread(TClient *client)
{
	client->run();
	delete client;
	return 0;
}

void accepted(int fd, struct sockaddr_in *cli_addr)
{
	BString name;
	uint8 *ip = (uint8 *)&cli_addr->sin_addr.s_addr;
	name << (int)ip[0] << "." << (int)ip[1] << "." << (int)ip[2] << "." << (int)ip[3];
	
	TClient *client = new TClient(fd, cli_addr, &config);
	thread_id id = spawn_thread(_accepted_thread, name.String(), B_NORMAL_PRIORITY, client);
	if (id < 0) {
		fprintf(stderr, "spawn_thread() failed.\n");
		delete client;
		close(fd);
		return;
	}
	
	if (resume_thread(id) != B_OK) {
		fprintf(stderr, "resume_thread() failed.\n");
		delete client;
		close(fd);
		return;
	}
}

#else
// other plathome ---------------------------------------------------------------
void accepted(int fd, struct sockaddr_in *cli_addr)
{
	// fix me
}

// ------------------------------------------------------------------------------
#endif

int start_server()
{
	BString names;
	config.ShareNames(&names);
	BString bstr(config.Comment());
	bstr << "," << names;
	if (config.IsBrowsable()) start_broadcasting(bstr.String());
	
	listening = 1;
	int server_fd, new_client_fd, clen;
	struct sockaddr_in serv_addr, cli_addr;
	
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	serv_addr.sin_port = htons(NETBFS_TCP_PORT);
	
	server_fd = socket(AF_INET, SOCK_STREAM, 0);
	if (server_fd == -1) {
		fprintf(stderr, "Can't start server. socket() failed. %s\n", strerror(errno));
		return -1;
	}
	
	if (bind(server_fd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) == -1) {
		fprintf(stderr, "Can't start server. bind() failed. %s\n", strerror(errno));
		return -2;
	}
	
	if (listen(server_fd, 5) == -1) {
		fprintf(stderr, "Can't start server. listen() failed. %s\n", strerror(errno));
		return -3;
	}
	
	signal(SIGINT, signal_handler);
	
	while(listening) {
		clen = sizeof(cli_addr);
		new_client_fd = accept(server_fd, (struct sockaddr *)&cli_addr, &clen);
		if (new_client_fd == -1) {
			fprintf(stderr, "Can't accept. %s\n", strerror(errno));
			continue;
		}
		accepted(new_client_fd, &cli_addr);
	}
	
	close(server_fd);
	if (config.IsBrowsable()) stop_broadcasting();
	
	fprintf(stderr, "Terminating.\n");
	snooze(1000000);
	
	return 0;
}
