#include "config.h"
#include "global.h"
#include "client.h"
#include "nodemon.h"


#ifdef _OSTYPE_BEOS
// BeOS -------------------------------------------------------------------------

#include <Looper.h>
#include <Messenger.h>
#include <Message.h>
#include <String.h>
#include <NodeMonitor.h>

enum {
	NODEMON_REGIST = 'nmrg',
	NODEMON_NOTIFY = 'nmnt',
};

#define NODEMON_SERVER "application/x-vnd.netbfs-nodemon_server"

class TNodeMonLooper : public BLooper
{
private:
	FILE *fout;
public:
	TNodeMonLooper(FILE *s_out, const char *name) : BLooper(name, B_LOW_PRIORITY) {
		fout = s_out;
	};
	~TNodeMonLooper() {};
	void Notify(BMessage *msg) {
		int32 op;
		ino_t from_directory, to_directory, node;
		const char *name;
		if ((msg->FindInt32("opcode", &op) != B_OK) ||
		    (msg->FindInt64("node", &node) != B_OK)) return;
		switch(op) {
			case B_ENTRY_CREATED:
				if ((msg->FindString("name", &name) != B_OK) ||
				    (msg->FindInt64("directory", &from_directory) != B_OK)) return;
				printf("C %d %d %s", from_directory, node, name);
				break;
			case B_ENTRY_REMOVED:
				if (msg->FindInt64("directory", &from_directory) != B_OK) return;
				printf("R %d %d", from_directory, node, name);
				break;
			case B_ENTRY_MOVED:
				if ((msg->FindString("name", &name) != B_OK) ||
				    (msg->FindInt64("to directory", &to_directory) != B_OK) ||
				    (msg->FindInt64("from directory", &from_directory) != B_OK)) return;
				printf("M %d %d %d", from_directory, to_directory, node, name);
				break;
			case B_STAT_CHANGED:
				if (msg->FindInt64("directory", &from_directory) != B_OK) return;
				printf("S %d %d", from_directory, node);
				break;
			case B_ATTR_CHANGED:
				if (msg->FindInt64("directory", &from_directory) != B_OK) return;
				printf("A %d %d", from_directory, node);
				break;
		}
		fflush(fout);
	};
	void MessageReceived(BMessage *msg) {
		switch(msg->what) {
			case NODEMON_NOTIFY:
				Notify(msg);
				break;
			default: BLooper::MessageReceived(msg);
		}
	};
};

TNodeMonLooper *node_looper = NULL;
thread_id node_looper_id;

int start_node_monitor(FILE *s_out, const char *dirpath, const char *name)
{
	BMessage msg(NODEMON_REGIST);
	BMessenger self(node_looper);
	msg.AddString("dirpath", dirpath);
	msg.AddMessenger("reply", self);
	BMessenger(NODEMON_SERVER).SendMessage(&msg);
	
	if (!node_looper) {
		node_looper = new TNodeMonLooper(s_out, name);
		node_looper_id = node_looper->Run();
		LOG << "spawn node monitor thread. id=" << node_looper_id << "  name=" << name << "\n";
	}
	return (node_looper > 0) ? 0 : -1;
}

void stop_node_monitor()
{
	if (node_looper) delete node_looper;
}

#else
// other plathome -------------------------------------------------------------

int start_node_monitor(FILE *s_out, const char *dirpath, const char *name)
{
	LOG << "Node monitor not supported.\n";
	return -1;
}

void stop_node_monitor()
{
}


#endif
