#ifndef _NETBFS_H
#define _NETBFS_H

#include <bone/sys/socket_module.h>
#include <bone/bone_endpoint.h>
#include <SupportDefs.h>

#define NETBFS_COMMAND_HEADER 0xbe
#define NETBFS_REPLY_HEADER 0xf5
#define NETBFS_DATA_MAXSIZE 65536
#define NETBFS_TCP_PORT 4989

enum netbfs_cmd {
	NETBFS_CMD_LOGIN = 0,
	NETBFS_CMD_START_NODEMON = 1,
	NETBFS_CMD_START_LIVEQUERY = 2,
	NETBFS_CMD_READDIR = 3,
	NETBFS_CMD_FOPEN = 4,
	NETBFS_CMD_FREAD = 5,
};

enum reply_status {
	NETBFS_OK = 0,
	NETBFS_ENOENT = 2,
	NETBFS_ENOMEM = 12,
	NETBFS_EACCES = 13,
	NETBFS_EINVAL = 22,
};

enum data_stat {
	NETBFS_UNCOMPRESSED = 0,
	NETBFS_LZO_COMPRESSED = 1
};

typedef struct netbfs_cmd_t {
	uint8 c_header;				// 0xbe
	uint8 c_command;
	uint32 c_data_length;
	uint8 c_data_stat;
	uint8 c_continuation;
} netbfs_cmd_t;

typedef struct netbfs_reply_t {
	uint8 r_header;				// 0xf5
	uint8 r_status;
	uint32 r_data_length;
	uint8 r_data_stat;
	uint8 r_continuation;
} netbfs_reply_t;

typedef struct fstat_t {
	ino_t fs_ino;
	mode_t fs_mode;
	off_t fs_size;
	time_t fs_atime;
	time_t fs_mtime;
	time_t fs_ctime;
	time_t fs_crtime;
	uint32 fs_namelen;
} fstat_t;

typedef struct fopen_t {	
	int fo_omode;
	uint32 fo_namelen;
} fopen_t;

typedef struct fio_t {
	off_t fio_pos;
	size_t fio_len;
} fio_t;



typedef struct netbfs_parms_t {
	struct sockaddr_in server;	// 接続先サーバー
	bigtime_t timeout;			// タイムアウト
	char idpass[64];			// username@password@sharename
	char null;
} netbfs_parms_t;


#endif
